import ReadingProgress from "@components/ReadingProgress.svelte";
import { Event, EventEmitter } from "@utils/event";
import { App, Modal } from "obsidian";
import { mount, unmount } from "svelte";

class SubmitEvent extends Event {
	constructor(public readonly pageNumber: number) {
		super();
	}
}

interface ReadingProgressModalEventMap {
	submit: SubmitEvent;
}

export class ReadingProgressModal extends EventEmitter<
	ReadingProgressModalEventMap,
	typeof Modal
>(Modal) {
	private component: ReturnType<typeof ReadingProgress> | undefined;

	constructor(app: App, private readonly pageLength: number) {
		super(app);
	}

	onOpen(): void {
		this.component = mount(ReadingProgress, {
			target: this.contentEl,
			props: {
				pageLength: this.pageLength,
				onSubmit: (pageNumber: number) => {
					this.emit("submit", new SubmitEvent(pageNumber));
				},
			},
		});
	}

	onClose(): void {
		if (this.component) {
			unmount(this.component);
			this.component = undefined;
		}
	}

	static createAndOpen(app: App, pageLength: number): Promise<number> {
		return new Promise((resolve) => {
			const modal = new ReadingProgressModal(app, pageLength);
			modal.once("submit", (event: SubmitEvent) => {
				modal.close();
				resolve(event.pageNumber);
			});
			modal.open();
		});
	}
}
